# Mode: Python; indent-tabs-mode: nil; tab-width: 4 -*-
# -*- coding: utf-8 -*-

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from collections import namedtuple, OrderedDict
import os
import re
import shutil
import signal
import syslog
import subprocess

import debconf

from ubiquity import plugin
from ubiquity import parted_server
from ubiquity import misc
from ubiquity import osextras
from ubiquity.install_misc import archdetect

import tempfile
import string

NAME = "educational"
AFTER = ["prepare", None]
BEFORE = "partman"
WEIGHT = 12

class PageGtk(plugin.PluginUI):
    plugin_title = "ubiquity/text/educational_heading_label"
    def __init__(self, controller, *args, **kwargs):
        self.controller = controller
        from gi.repository import Gtk

        distro = os.popen('/usr/lib/ubiquity/le-utils/le-env').readline()
        if distro == 'minicom':
            self.page = None
            return

        builder = Gtk.Builder()
        self.controller.add_builder(builder)
        builder.add_from_file(os.path.join(os.environ["UBIQUITY_GLADE"], "stepEducational.ui"))
        builder.connect_signals(self)
        self.page = builder.get_object("stepEducational")
        self.educational_heading = builder.get_object("educational_heading")
        self.educational_notification_box = builder.get_object("educational_notification_box")
        self.educational_button_start = builder.get_object("educational_button_start")
        self.educational_button_cancel = builder.get_object("educational_button_cancel")
        self.educational_warning_box = builder.get_object("educational_warning_box")
        self.educational_warning_message = builder.get_object("educational_warning_message")
        self.educational_educational_progress_bar = builder.get_object("educational_progress_bar")

        self.plugin_widgets = self.page

    def educational_button_start_activate (self, unused_widget):
        found = False
        subp = subprocess.Popen(["cat", "/proc/partitions"],stdout=subprocess.PIPE)
        for line in subp.communicate()[0].splitlines():
            device = re.split(" +", line)
            if len(device) > 4:
                device = os.path.realpath("/dev/%s" % device[4])
                usage = None
                is_partition = False
                mount = False
                subp = subprocess.Popen(["udevadm", "info", "-q", "property","-n", device],stdout=subprocess.PIPE)
                for line in subp.communicate()[0].splitlines():
                    line = line.strip()
                    if line == "DEVTYPE=partition":
                        is_partition = True
                    elif line.startswith("ID_FS_TYPE="):
                        usage = line[11:]
                        if usage != "swap":
                            mount = True
                if is_partition and mount:
                    if misc.is_removable(device):
                        subp = subprocess.Popen(["mount"], stdout=subprocess.PIPE).communicate()[0]
                        if device in subp:
                            mount_path = subp[subp.index(device):]
                            mount_path = mount_path[string.find(mount_path, "on ")+3:string.find(mount_path, " type ")]
                        else:
                            mount_path = tempfile.mkdtemp()
                        if mount_path != "/cdrom":
                            try:
                                subprocess.check_call(["mount", device, mount_path])
                                self.debug(unicode("educational destination %s" % mount_path, "utf8"))
                                subp = subprocess.Popen(["df", "--block-size=1", device], stdout=subprocess.PIPE).communicate()[0]
                                if device in subp:
                                    freeSpace = subp[subp.index(device):].split()[3]
                                    self.debug("educational free space on destination %s" % freeSpace)
                                    if int(freeSpace) > int(self.educationalSize):
                                        found = 1
                                        self.controller.dbfilter.preseed("ubiquity/educational_backup", device)
                                        self.educational_warning_box.hide()
                                        self.educational_button_cancel.set_visible(True)
                                        self.educational_button_start.set_sensitive(False)
                                        self.educational_educational_progress_bar.set_visible(True)
                                        self.educational_educational_progress_bar.set_fraction(0)
                                        for partition in self.partitions:
                                            if self.size > self.educationalSize:
                                                break
                                            educationalPath = "%s/home/ConteudoMEC" % partition
                                            index = len(partition) + 5#len("/home")
                                            self.debug("educational source %s" % educationalPath)
                                            self.files = []
                                            self.dirs = []
                                            self.dirs.append(educationalPath)
                                            self.size = self.size + os.path.getsize(educationalPath)
                                            self.search(mount_path, index)
                                            found = self.progress(mount_path, index)
                                        self.debug("educational total size copied %s" % self.size)
                            except Exception, e:
                                self.debug(unicode("educational exception: %s" % e, "utf8"))
                            finally:
                                self.debug("educational unmounting device %s" % device)
                                subprocess.call(["umount", "-l", mount_path])
                                self.reset_page()
                                if found == 1:
                                    self.controller.go_forward()
                                    break
        if found == 0:
            self.educational_warning_box.show_all()

    def search(self, mount_path, index):
        for dir in self.dirs:
            try:
                subprocess.call("sudo mkdir -p \"%s%s\"" % (mount_path, dir[index:]), shell=True)
                subp = subprocess.Popen(["ls", dir], stdout=subprocess.PIPE).communicate()[0]
                for line in subp.splitlines():
                    path = dir + "/" + line
                    if os.path.isdir(path):
                        self.size = self.size + os.path.getsize(path)
                        self.dirs.append(path)
                        self.debug(unicode("educational checking path %s %s" % (path, self.size), "utf8"))
                    elif os.path.isfile(path):
                        self.files.append(path)
            except Exception, e:
                self.debug(unicode("educational exception: %s" % e, "utf8"))

    def progress(self, mount_path, start):
        for file in self.files:
            if self.size > self.educationalSize:
                return 2
            self.educational_educational_progress_bar.set_fraction(float(self.size) / self.educationalSize)
            self.controller.allow_change_step(False)
            end = string.rfind(file, "/")
            cp = subprocess.Popen("sudo cp -f \"%s\" \"%s%s\"" % (file, mount_path, file[start:end]), shell=True)
            cp.wait()
            self.size = self.size + os.path.getsize(file)
        return 1

    def educational_button_cancel_activate (self, unused_widget):
        self.size = self.educationalSize + 1

    def reset_page(self):
        self.educational_button_cancel.set_visible(False)
        self.educational_button_start.set_sensitive(True)
        self.educational_educational_progress_bar.set_visible(False)
        self.controller.allow_change_step(True)

    def plugin_on_next_clicked(self):
        self.leave_page()

    def plugin_on_back_clicked(self):
        self.leave_page()
        
    def leave_page(self):
        for partition in self.partitions:
            self.debug("educational unmounting %s" % partition)
            subprocess.call(["umount", "-l", partition])
        self.debug("Leaving step educational")

class PageKde(plugin.PluginUI):
    plugin_breadcrumb = "ubiquity/text/educational_heading_label"
    def __init__(self, controller, *args, **kwargs):
        self.controller = controller
        try:
            from PyQt4 import uic
            self.page = uic.loadUi("/usr/share/ubiquity/qt/stepEducational.ui")
        except Exception, e:
            print >>sys.stderr,"Could not create prepare page:", str(e)
            self.debug("Could not create prepare page: %s", e)
        self.plugin_widgets = self.page

class Page(plugin.Plugin):
    def prepare(self):
        self.debug("Entering step educational")
        self.ui.educational_notification_box.hide()
        self.ui.educational_button_start.set_visible(False)
        self.preseed("ubiquity/educational_backup", "")
        self.ui.partitions = []
        self.ui.educationalSize = 0
        self.ui.size = 0
        heading = self.calculate_educational_heading()
        if heading == "":
            self.ui.controller.go_forward()
        else:
            self.ui.educational_heading.set_label(heading)
            q = "ubiquity/educational_warning"
            self.db.subst(q, "SIZE", misc.format_size(self.ui.educationalSize))
            self.ui.educational_warning_message.set_text(self.description(q))
            self.ui.educational_notification_box.show_all()
            self.ui.educational_button_start.set_visible(True)
        return plugin.Plugin.prepare(self)

    def calculate_educational_heading(self):
        subp = subprocess.Popen(["cat", "/proc/partitions"],stdout=subprocess.PIPE)
        for line in subp.communicate()[0].splitlines():
            device = re.split(" +", line)
            if len(device) > 4:
                device = os.path.realpath("/dev/%s" % device[4])
                usage = None
                is_partition = False
                mount = False
                subp = subprocess.Popen(["udevadm", "info", "-q", "property","-n", device],stdout=subprocess.PIPE)
                for line in subp.communicate()[0].splitlines():
                    line = line.strip()
                    if line == "DEVTYPE=partition":
                        is_partition = True
                    elif line.startswith("ID_FS_TYPE="):
                        usage = line[11:]
                        if usage != "swap":
                            mount = True
                if is_partition and mount:
                    if not misc.is_removable(device):
                        mount_path = tempfile.mkdtemp()
                        try:
                            subprocess.check_call(["mount", device, mount_path])
                            educationalPath = "%s/home/ConteudoMEC" % mount_path
                            self.debug("educational device %s" % device)
                            if os.path.exists(educationalPath):
                                self.debug("educational has content %s" % educationalPath)
                                subp = subprocess.Popen(["du", "-bs", educationalPath], stdout=subprocess.PIPE).communicate()[0]
                                self.ui.educationalSize = self.ui.educationalSize + int(subp.split()[0])
                                self.ui.partitions.append(mount_path)
                            else:
                                subprocess.call(["umount", "-l", mount_path])
                        except Exception, e:
                            self.debug(unicode("educational exception: %s" % e, "utf8"))
        self.debug("educational content size %s" % self.ui.educationalSize)
        if self.ui.educationalSize == 0:
            return ""
        else:
            q = "ubiquity/educational_heading"
            self.db.subst(q, "SIZE", misc.format_size(self.ui.educationalSize))
            return self.description(q)

